package com.wang.mqtt;

import android.app.Service;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.drawable.Drawable;
import android.os.IBinder;
import android.widget.Button;
import android.widget.Toast;

import org.eclipse.paho.android.service.MqttAndroidClient;
import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.IMqttToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

public class MqttService extends Service {
    private Callback callback;

    static MqttAndroidClient mqttAndroidClient;

    public static String hostName;
    public static String port;
    public static String serverUri = "tcp://" + hostName + ":" + port;
    public static String userName;
    public static String passWord;
    public static String clientId = "ExampleAndroidClient";
    public static String subscriptionTopic;
    public static String publishonTopic;
    public static Integer subQos;
    public static Integer pubQos;

    private boolean isConnect = false;

    public MqttService() {
    }

    @Override
    public void onCreate() {
            init();
            System.out.println("初始化~~~~");
        super.onCreate();
    }

    @Override
    public void onDestroy() {
        stopSelf();
        try {
            isConnect = !isConnect;
//            disconnect();
            mqttAndroidClient.unregisterResources();
            mqttAndroidClient.disconnect();
            mqttAndroidClient = null;
            System.out.println("MQTT关闭服务！");
            makeToast("断开连接！");
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }

    public void init(){
        //URI
        serverUri = "tcp://" + hostName + ":" + port;

        //新建client实例
        clientId = clientId + System.currentTimeMillis();
        mqttAndroidClient = new MqttAndroidClient(this,serverUri,clientId);
        //设置回调函数
        mqttAndroidClient.setCallback(new MqttCallbackExtended() {
            @Override
            public void connectComplete(boolean reconnect, String serverURI) {

            }

            @Override
            public void connectionLost(Throwable cause) {

            }

            @Override
            public void messageArrived(String topic, MqttMessage message) throws Exception {
                String str = new String(message.getPayload());
                //将收到的消息弹出
//                makeToast(str);

                //接收消息转成字符串
                String[] tempHumidity = receiveMessage(str);
                if (callback != null){
                    callback.onMessageChange(tempHumidity[0]);
                }
                if (callback != null){
                    callback.onMessageChange(tempHumidity[1]);
                }
//                makeToast(tempHumidity[0]);
//                makeToast(tempHumidity[1]);

            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken token) {

            }
        });

        //连接设置
        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        //设置自动重连
        mqttConnectOptions.setAutomaticReconnect(false);
        //清理session缓存
        mqttConnectOptions.setCleanSession(false);
        //设置超时时间 单位：秒
        mqttConnectOptions.setConnectionTimeout(10);
        // 心跳包发送间隔，单位：秒
        mqttConnectOptions.setKeepAliveInterval(30);
        //设置用户名
        mqttConnectOptions.setUserName(userName);
        //设置密码
        mqttConnectOptions.setPassword(passWord.toCharArray());

        //连接Mqtt服务器
        try {
            isConnect = !isConnect;
            mqttAndroidClient.connect(mqttConnectOptions, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    makeToast("连接成功！");
                    //订阅话题
                    setSubscriptionTopic();
                    SettingFragment.btnLogoutFlag = true;
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    makeToast("连接失败！");
                }
            });
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    public static void disconnect() {
        try {
            if (mqttAndroidClient != null) {
//                if (mqttAndroidClient.isConnected())
//                    mqttAndroidClient.unsubscribe();
                mqttAndroidClient.close();
                mqttAndroidClient.unregisterResources();
                mqttAndroidClient = null;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 订阅话题
     */
    public void setSubscriptionTopic(){
        try {
            mqttAndroidClient.subscribe(subscriptionTopic, subQos, null, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken asyncActionToken) {
                    makeToast("成功订阅到" + subscriptionTopic);
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                    makeToast("订阅失败");
                }
            });
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return new Binder();
    }

    public class Binder extends android.os.Binder{
        public MqttService getService(){
            return MqttService.this;
        }
    }

    public void setCallback(Callback callback){
        this.callback = callback;
    }

    public interface Callback{
        void onMessageChange(String message);
    }

    public void makeToast(String message){
        Toast.makeText(this,message,Toast.LENGTH_SHORT).show();
    }

    public static void publish(String msg){


        Boolean retained = false;
        try {
            if (mqttAndroidClient != null){
                mqttAndroidClient.publish(publishonTopic, msg.getBytes(), pubQos.intValue(), retained.booleanValue());
            }
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    public String[] receiveMessage(String str){
        String[] tempHumidity = new String[2];

        //去掉所有空格
        str = str.replaceAll(" ","");
        //去掉花括号
        str = str.substring(1,str.length()-1);
        //逗号分割，分出每一对键值对
        String[] strs = str.split(",");
        if(strs.length == 2){
            //冒号分割，将温度键值对分开
            String[] temp = strs[0].split(":");
            //去掉双引号，即为温度值

            //判断key值是否为Temperature
            if (temp[0].substring(1,temp[0].length()-1).equals("Temperature")){
                tempHumidity[0] = temp[1].substring(1,temp[1].length()-1);
                tempHumidity[0] = tempHumidity[0] + "℃";
            }else {//如果不是则保持原来的值
                tempHumidity[0] = "keepOldValue";
            }

            //冒号分割，将湿度键值对分开
            String[] humidity = strs[1].split(":");
            //去掉双引号，即为湿度值

            //判断key值是否为Humidity
            if (humidity[0].substring(1,humidity[0].length()-1).equals("Humidity")){
                tempHumidity[1] = humidity[1].substring(1,humidity[1].length()-1);
                tempHumidity[1] = tempHumidity[1] + "%";
            }else {//如果不是则保持原来的值
                tempHumidity[1] = "keepOldValue";
            }
        }

        return tempHumidity;
    }
}
